package ij.plugin;
import ij.*;
import ij.gui.*;
import ij.process.*;

public class ImageCalculator implements PlugIn {

	private static int operator;
	private static String srcTitle = "";
	private static String dstTitle = "";
	//private static double scale = 1.0;
	//private static double offset = 0.0;
	

	public void run(String arg) {
		int[] wList = WindowManager.getIDList();
		if (wList==null) {
			IJ.error("No windows are open.");
			return;
		}
		IJ.register(ImageCalculator.class);
		String[] titles = new String[wList.length];
		String[] operators = {"Add","Subtract","Multiply","Divide", "AND", "OR", "XOR", "Min", "Max", "Average", "Difference", "Copy"};
		for (int i=0; i<wList.length; i++) {
			ImagePlus imp = WindowManager.getImage(wList[i]);
			if (imp!=null)
				titles[i] = imp.getTitle();
			else
				titles[i] = "";
		}
		GenericDialog gd = new GenericDialog("Image Calculator", IJ.getInstance());
		String defaultItem;
		if (srcTitle.equals(""))
			defaultItem = titles[0];
		else
			defaultItem = srcTitle;
		gd.addChoice("Source:", titles, defaultItem);
		gd.addChoice("Operation:", operators, operators[operator]);
		if (dstTitle.equals(""))
			defaultItem = titles[0];
		else
			defaultItem = dstTitle;
		gd.addChoice("Destination:", titles, defaultItem);
		//gd.addNumericField("Scale:", scale, 1);
		//gd.addNumericField("Offset:", offset, 0);
		gd.showDialog();
		if (gd.wasCanceled())
			return;
		int srcIndex = gd.getNextChoiceIndex();
		srcTitle = titles[srcIndex];
		operator = gd.getNextChoiceIndex();
		int dstIndex = gd.getNextChoiceIndex();
		//scale = gd.getNextNumber();
		//offset = gd.getNextNumber();
		dstTitle = titles[dstIndex];
		ImagePlus src = WindowManager.getImage(wList[srcIndex]);
		ImagePlus dst = WindowManager.getImage(wList[dstIndex]);
		int srcSize = src.getStackSize();
		int dstSize = dst.getStackSize();
		if (srcSize>1 && dstSize>1 && srcSize!=dstSize) {
			IJ.showMessage("Image Calculator", "Both stacks must have the same number of sizes.");
			return;
		}
		if (dstSize>1 && (srcSize==1||srcSize==dstSize)) {
			YesNoCancelDialog d = new YesNoCancelDialog(IJ.getInstance(),
				"Process Stacks?", "Process all "+dstSize+" slices?");
			if (d.cancelPressed())
				return;
			if (d.yesPressed())
				doStackOperation(src, dst);
			else
				doOperation(src, dst);
		} else
			doOperation(src, dst);
	}
	
	void doStackOperation(ImagePlus src, ImagePlus dst) {
		int mode = getBlitterMode();
		ImageWindow win = dst.getWindow();
		if (win!=null)
			WindowManager.setCurrentWindow(win);
		Undo.reset();
		StackProcessor sp = new StackProcessor(dst.getStack(), dst.getProcessor());
		try {
			if (src.getStackSize()==1)
				sp.copyBits(src.getProcessor(), 0, 0, mode);
			else
				sp.copyBits(src.getStack(), 0, 0, mode);
		}
		catch (IllegalArgumentException e) {
			IJ.error("\""+src.getTitle()+"\": "+e.getMessage());
			return;
		}
		dst.updateAndDraw();
	}

	void doOperation(ImagePlus src, ImagePlus dst) {
		int mode = getBlitterMode();
		ImageProcessor srcIP = src.getProcessor();
		ImageProcessor dstIP = dst.getProcessor();
		ImageWindow win = dst.getWindow();
		if (win!=null)
			WindowManager.setCurrentWindow(win);
		dstIP.snapshot();
		Undo.setup(Undo.FILTER, dst);
		try {
			dstIP.copyBits(srcIP, 0, 0, mode);
		}
		catch (IllegalArgumentException e) {
			IJ.error("\""+src.getTitle()+"\": "+e.getMessage());
			return;
		}
		dst.updateAndDraw();
	}

	private int getBlitterMode() {
		int mode=0;
		switch (operator) {
			case 0: mode = Blitter.ADD; break;
			case 1: mode = Blitter.SUBTRACT; break;
			case 2: mode = Blitter.MULTIPLY; break;
			case 3: mode = Blitter.DIVIDE; break;
			case 4: mode = Blitter.AND; break;
			case 5: mode = Blitter.OR; break;
			case 6: mode = Blitter.XOR; break;
			case 7: mode = Blitter.MIN; break;
			case 8: mode = Blitter.MAX; break;
			case 9: mode = Blitter.AVERAGE; break;
			case 10: mode = Blitter.DIFFERENCE; break;
			case 11: mode = Blitter.COPY; break;
		}
		return mode;
	}

}
