package ij.plugin.filter;
import java.awt.Color;
import ij.*;
import ij.gui.*;
import ij.process.*;
import ij.plugin.filter.ParticleAnalyzer;
import ij.measure.*;

/** Implements ImageJ's Analyze/Analyze Line Graph command. */
public class LineGraphAnalyzer implements PlugInFilter, Measurements  {

	public int setup(String arg, ImagePlus imp) {
		return DOES_8G+NO_CHANGES;
	}

	public void run(ImageProcessor ip) {
		analyze((ByteProcessor)ip);
	}
	
	/** Uses ImageJ's particle analyzer to extract a set
		of coordinate pairs from a digitized line graph. */
	public void analyze(ByteProcessor ip) {
		int width = ip.getWidth();
		int height = ip.getHeight();
		ImageProcessor ip2 = new ByteProcessor(width, height, (byte[])ip.getPixels(), null);
		ip2.setColor(Color.white);
		for (int i=1; i<width; i+=2) {
			ip2.moveTo(i,0);
			ip2.lineTo(i,height-1);
		}
		ip2 = ip2.rotateRight();
		ImagePlus imp2 = new ImagePlus("Temp", ip2);
		if (IJ.altKeyDown()) imp2.show();
		
		int options = ParticleAnalyzer.SHOW_PROGRESS;
		int measurements = CENTROID;
		int minSize = 1;
		int maxSize = 999999;
		ParticleAnalyzer pa = new ParticleAnalyzer(options,measurements,minSize,maxSize);
		pa.analyze(imp2);
		ResultsTable rt = pa.getResultsTable();
		float[] y = rt.getColumn(1);				
		float[] x = rt.getColumn(2);
		new PlotWindow("Line Graph", "X", "Y", x, y).draw();				
	}
	
}
