import ij.*;
import ij.plugin.PlugIn;
import ij.io.*;
import java.awt.*;
import java.io.*;

public class AHFWriter implements PlugIn {

	private ImagePlus imp;
	private static String defaultDirectory = null;

	public void run(String arg) {
		imp = WindowManager.getCurrentImage();
		if (imp==null)
			{IJ.noImage(); return;}
		saveAsAHF();
	}

	/** Save the image in AHF format using a save file
		dialog. Returns false if the user selects cancel. */
	public boolean saveAsAHF() {
		String path = getPath("ASCII Header Format", ".ahf");
		if (path==null)
			return false;
		else
			return saveAsAHF(path);
	}
	
	/** Save the image in AHF format using the specified path. Returns
		false if there is an I/O error. */
	public boolean saveAsAHF(String path) {
		FileInfo fi = imp.getFileInfo();
		try {
			AHFEncoder encoder = new AHFEncoder(fi);
			DataOutputStream out = new DataOutputStream(new BufferedOutputStream(new FileOutputStream(path)));
			encoder.write(out);
			out.close();
		}
		catch (IOException e) {
			IJ.error("An error occured writing the file.\n \n" + e);
			return false;
		}
		imp.changes = false;
		return true;
	}

	String getPath(String type, String extension) {
		FileDialog fd = new FileDialog(IJ.getInstance(), "Save as "+type+"...", FileDialog.SAVE);
		String name = imp.getTitle();
		int dotIndex = name.lastIndexOf(".");
		if (dotIndex>=0)
			name = name.substring(0, dotIndex)+extension;
		fd.setFile(name);

		if (defaultDirectory!=null)
			fd.setDirectory(defaultDirectory);
		fd.setVisible(true);
		name = fd.getFile();
		String directory = fd.getDirectory();
		defaultDirectory = directory;
		fd.dispose();
		if (name == null)
			return null;
		IJ.showStatus("Saving as "+ type + ": " + directory + name);
		return directory+name;
	}

}


/**Saves an image described by a FileInfo object in AHF (ASCII Header File) format.*/
class AHFEncoder {

	private FileInfo fi;
	
	public AHFEncoder (FileInfo fi) {
		this.fi = fi;
	}
	
	/** Saves the image as a AHF file. */
	public void write(DataOutputStream out) throws IOException {
		String header;
		header =	"AHF{\n"
					+"pixels{"+fi.width+"}\n"
					+"lines{"+fi.height+"}\n"
					+"}";
		out.writeBytes(header);
		fi.offset = header.length();
		new ImageWriter(fi).write(out);
	}
	
}

